/*
	File:		Register.h

	Function:	Class registry

	Author:		Andrew Willmott
*/

#ifndef __Register__
#define __Register__

#include "cl/Basics.h"
#include "cl/Array.h"

typedef UInt32 ClassID;
typedef Void *(*ObjectCreator)();

struct RegistryEntry
{
	ClassID			baseID;
	ClassID			id;
	ObjectCreator	creator;
	const Char		*name;
	const Char		*description;
};

typedef Array<RegistryEntry>	ClassTable;

struct BaseEntry
{
	ClassID		baseID;
	ClassTable	table;
};

typedef Array<BaseEntry>		BaseTable;

class Register
{
public:
	Register(
		ClassID         id,       
		ObjectCreator   creator,       
		const           Char *name = "untitled",    
		const           Char *description = 0
	) { RegisterClass('none', id, creator, name, description); };
	
	Register(
		ClassID			baseID, 
		ClassID         id,       
		ObjectCreator   creator,       
		const           Char *name = "untitled",    
		const           Char *description = 0
	) { RegisterClass(baseID, id, creator, name, description); };


	static Void		RegisterClass(
		ClassID			baseID, 
		ClassID         id,       
		ObjectCreator   creator,       
		const           Char *name = "untitled",    
		const           Char *description = 0
	);

	static Void		*MakeObject(ClassID id);
	static Void		*MakeObject(const Char *name);
	static Void		*MakeObject(ClassID baseID, ClassID id);
	static Void		*MakeObject(ClassID baseID, const Char *name);
	static Void		ListObjects();
	static Void		ListObjects(ClassID baseID);
	
protected:
	static ClassTable	*FindClassTable(ClassID baseID);
	static BaseTable	*table;
};

#define MAKE_REGISTRAR(REGISTRAR, BASE_ID)								\
	class REGISTRAR : public Register									\
	{																	\
	public:																\
		REGISTRAR(														\
			ClassID         id,											\
			ObjectCreator   creator,									\
			const           Char *name = "untitled",					\
			const           Char *description = 0						\
		) :	Register(BASE_ID, id, creator, name, description) {};		\
	}

#endif

