/* 
	Test program for random GCL stuff
	
*/

#include "gcl/XGraphicsSystem.h"
#include "gcl/EPSRenderer.h"
#include "gcl/MesaRenderer.h"
#include "gcl/Readers.h"
#include <stdlib.h>
#include "gcl/SceneLang.h"

static Void CubeScene(Renderer &pane)
{
	pane
		.SetTransform(Rotation(vl_y, vl_pi / 2.0))
		.Begin(renPoly)
		.SetColour(cRed)
		.SetPoint(Point(-1, -1, 1))
		.SetPoint(Point(-1,  1, 1))
		.SetPoint(Point( 1,  1, 1))
		.SetPoint(Point( 1, -1, 1))
		.End();
		
	pane
		.Push()
		.SetTransform(Rotation(vl_y, vl_pi / 2.0))
		.Begin(renPoly)
		.SetColour(cBlue)
		.SetPoint(Point(-1, -1, 1))
		.SetPoint(Point(-1,  1, 1))
		.SetPoint(Point( 1,  1, 1))
		.SetPoint(Point( 1, -1, 1))
		.End();
	
	pane
		.SetTransform(Rotation(vl_y, vl_pi / 2.0))
		.Begin(renPoly)
		.SetColour(cOrange)
		.SetPoint(Point(-1, -1, 1))
		.SetPoint(Point(-1,  1, 1))
		.SetPoint(Point( 1,  1, 1))
		.SetPoint(Point( 1, -1, 1))
		.End();

	pane
		.Pop()
		.SetTransform(Rotation(vl_x, vl_pi / 2.0))
		.Begin(renPoly)
		.SetColour(cGreen)
		.SetPoint(Point(-1, -1, 1))
		.SetPoint(Point(-1,  1, 1))
		.SetPoint(Point( 1,  1, 1))
		.SetPoint(Point( 1, -1, 1))
		.End();
		
	pane	
		.Pop()
		.Show();
}

Void ImageTest(Renderer &myPane1)
{
#if 0
	myPane1.GetImage(image);
	
	
	cout << "grabbed image of " << image.Rows() << " x " << image.Cols() << endl;
	cout << "starting pane 2" << endl;
	
	myPane2.Clear().SetImage(image);
	myPane2.Show();
		
#if 0
	Pixel::currentChannel = chRed;
	image.MakeBlock(1);
	myPane1.Clear().Image(image);
	myPane1.Show();
	Pixel::currentChannel = chBlue;
	image.MakeBlock(0.5);
	myPane1.Clear();
	myPane1.SetImage(image);
	myPane1.Show();
	Pixel::currentChannel = chGreen;
	image.MakeBlock(0.2);
	myPane1.Clear();
	myPane1.SetImage(image);
	myPane1.Show();
	
	blob.SetSize(50, 50);
	blob.MakeUnit(1);
	sub(image, 30, 40, 50, 50) = blob;
	
	myPane1.Clear();
	myPane1.SetImage(image);
	myPane1.Show();
		
	gs.Run();
#endif
#endif
}

Void GouraudTest(Renderer *r)
{
	r->Clear().Show();
	r->Begin(renPoly);
	r->SetColour(cRed).SetPoint(Point(-1, -1, 0))
		.SetColour(cGreen).SetPoint(Point(1, -1, 0))
		.SetColour(cBlue).SetPoint(Point(1, 1, 0))
		.SetColour(cWhite).SetPoint(Point(-1, 1, 0));
	r->End();
	r->Show();
}

Void XWindowsTest()
{
	XGraphicsSystem			*xgs;
	XEventPane				*window;
	RGBAImage				image;
	Int						x;
	
	xgs = new XGraphicsSystem;
	xgs->DumpVisualInfo();
	window = new XEventPane;
	xgs->CreateWindow(window, "bob", 200, 200);
	image.SetSize(200, 200);
	image.Clear(cRed);
	window->PutImage(image);

	xgs->Run();
	window->Hide();
	xgs->Spin();
	cin >> x;
	window->Resize(400, 400);
	window->Show();
	window->PutImage(image);
	xgs->Run();
	window->Resize(600, 600);
	cin >> x;
	xgs->Run();
	
}

#include "gcl/Clipper.h"

class MyClip : public Clipper
{
public:
	Void ViewPoint(const HPoint &p, Bool draw);
};

Void MyClip::ViewPoint(const HPoint &p, Bool draw)
{
	if (draw)
		cout << "to " << p << endl;
	else
		cout << "from " << p << endl;
}

Void ClipTest()
{
	MyClip clipper;

	clipper.ClipPoint(HPoint(1.1, 2.2, 1.2, 1.0), false);
	clipper.ClipPoint(HPoint(2.0, 2.5, 1.5, 1.0), true);
	clipper.ClipPoint(HPoint(0.5, 0.5, 0.1, 1.0), true);
	clipper.ClipPoint(HPoint(0.4, -0.5, 0.1, 1.0), true);
	clipper.ClipPoint(HPoint(0.4, 0.5, -0.1, 1.0), true);
	cout << endl;
}

#define ON

main(int argc, char **argv)
{
	XGraphicsSystem			*xgs;

	EPSRenderer				*epsRenderer;
	MesaRenderer			*mesaRenderer;
	GLXPane					*xglRenderer;
	Renderer				*r;

	Camera					camera(false);
	Int						i;
	RGBAImage 				image;
	scScenePtr				scene;
	FileName				file;
	
	slInit();

//	XWindowsTest();
	ClipTest();


#ifdef OFF
	image.SetSize(400, 400);
	image.Clear(cGrey);
	mesaRenderer = new MesaRenderer;
	mesaRenderer->Init(image);
	r = mesaRenderer;
#endif

#ifdef OFF
	#define PAUSE gs.Run()
	xglRenderer = new GLXPane;
	gs.CreateWindow(xglRenderer, "blob", 400, 400);
	r = xglRenderer;
#else
	#define PAUSE 
#endif

#ifdef OFF
	image.SetSize(100, 100);
	image.Clear(cPurple);
	r->SetBgColour(cGrey);
	r->Clear();
	r->PutImage(image);
	r->Show();
	PAUSE;
#endif

#ifdef OFF
	image.SetSize(200, 200);
	image.Clear(cGreen);
	file.SetPath("green.ppm");
	image.Save(file);
#endif

#ifdef OFF
	image.Clear(cGrey75);
	for (i = 0; i < 20; i++)
	{
		PaintSky(image, Coord(0.45 - i / 20.0, 0.45 - i / 20.0));
		file.SetPath(String().Printf("sky-%04d.tif", i));
		image.Save(file);
	}
#endif
	
#ifdef OFF
//	file.SetPath("Black Sun:reactor:scenes:scomplex_09.sl");
	file.SetPath("/usr/people/ajw/reactor/exp/scenes/simple/complex_01_01.sl");
//	file.SetPath("/usr/people/ajw/reactor/rad/scenes/box.sl");
	if (!(scene = SceneReader::Load(file)))
		exit(1);

	r->SetBgColour(cGrey);
	r->Clear();
	r->Draw(*scene);
	r->Show();

//	r->GetImage(image);
	file.SetPath("test.ppm");
	image.Save(file);
	PAUSE;
#endif

#ifdef OFF
	r->SetBgColour(cYellow);
	r->Clear();
	camera.SetZoom(0.5);
	r->SetCamera(camera);
	r->SetTransform(Shift(Vector(0.3, 0.2, 0)) * Rotation(vl_z, DegsToRads(30)));
	GouraudTest(r);
	r->Show();
#endif	

#ifdef OFF
	r->SetBgColour(cCyan);
	r->Clear();
	camera.SetZoom(2);
	r->SetCamera(camera);
	CubeScene(*r);
	r->Show();
#endif	
		
}
