/*
	File:		MRFaceCluster.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/MRFaceCluster.h"
#include "gcl/Renderer.h"
#include "gcl/Draw.h"

FaceCluster::FaceCluster()
{
}


Void FaceCluster::DrawPlane(Renderer &r)
{
	Point		corners[8];
	Int			i;
		
	MakeBoxCorners(min, max, corners);	
	r.Push();
	r.SetTransform(trans(Rotation(axesQuat)));

	r.N(vl_z).Begin(renPoly)
		.P((corners[0] + corners[4]) * 0.5).P((corners[1] + corners[5]) * 0.5)
		.P((corners[2] + corners[6]) * 0.5).P((corners[3] + corners[7]) * 0.5).End();

	r.Pop();
}

Void FaceCluster::DrawBox(Renderer &r, Bool solid)
{
	Point		corners[8];
	Int			i;
		
	MakeBoxCorners(min, max, corners);	
	r.Push();
	r.SetTransform(trans(Rotation(axesQuat)));
	
	if (solid)
		PaintBox(r, corners);
	else
		FrameBox(r, corners);

	r.Pop();
}

Void FaceCluster::DrawSideAreas(Renderer &r)
{
	Point		corners[8];
	Int			i;
		
	MakeBoxCorners(min, max, corners);	
	r.Push();
	r.SetTransform(trans(Rotation(axesQuat)));

	Point	centre = 0.5 * (min + max);
	Vector	delta;
	
	delta = Vector(max[0], centre[1], centre[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[0] / 255.0)) * Shift(-delta));
	r.N(vl_x).Begin(renPoly)
		.P(corners[1]).P(corners[2]).P(corners[6]).P(corners[5]).End();
	r.Pop();

	delta = Vector(min[0], centre[1], centre[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[3] / 255.0)) * Shift(-delta));
	r.N(-Vector(vl_x)).Begin(renPoly)
		.P(corners[0]).P(corners[4]).P(corners[7]).P(corners[3]).End();
	r.Pop();

	delta = Vector(centre[0], max[1], centre[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[1] / 255.0)) * Shift(-delta));
	r.N(vl_y).Begin(renPoly)
		.P(corners[7]).P(corners[6]).P(corners[2]).P(corners[3]).End();
	r.Pop();

	delta = Vector(centre[0], min[1], centre[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[4] / 255.0)) * Shift(-delta));
	r.N(-Vector(vl_y)).Begin(renPoly)
		.P(corners[0]).P(corners[1]).P(corners[5]).P(corners[4]).End();
	r.Pop();

	delta = Vector(centre[0], centre[1], max[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[2] / 255.0)) * Shift(-delta));
	r.N(vl_z).Begin(renPoly)
		.P(corners[4]).P(corners[5]).P(corners[6]).P(corners[7]).End();
	r.Pop();

	delta = Vector(centre[0], centre[1], min[2]);
	r.Push().SetTransform(Shift(delta) * Scalef(sqrt(sideArea[5] / 255.0)) * Shift(-delta));
	r.N(-Vector(vl_z)).Begin(renPoly)
		.P(corners[3]).P(corners[2]).P(corners[1]).P(corners[0]).End();
	r.Pop();

	r.Pop();
}

Void FaceCluster::Draw(Renderer &r)
{
#if 0
	DrawPlane(r);
	r.SetColour(cGreen);
	r.N(vl_z);
	DrawBox(r, true);
#endif

	DrawBox(r, true);
}
