/*
	File:			Plot2D.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1998, Andrew Willmott

	Notes:			

*/

#include "gcl/Plot2D.h"

/** \class scPlot2D
	Defines a scene object which provides a 2D plot of a given
	function. Currently only over a rectangular domain.
*/

scPlot2D::scPlot2D() : scPrimitive(pPoly), plotType(plotStep)
{
	samples.SetSize(10, 10);
	samples.MakeZero();
}
 
const Char *scPlot2D::Label() const
{
	return("2D Plot");
}

Void scPlot2D::Draw(Renderer &r, SLContext *context)
{
	Int 		i, j;
	Point		pos0, pos1, pos2, pos3;
	Vector		dx, dz, normal;
	Colour		col = *(SC_GET(Colour));
				
	r.MakeCurrent();

	if (plotType != plotStep)
	{
		dx = Vector(2.0 / (samples.Cols() - 1), 0, 0);
		dz = Vector(0, 0, 2.0 / (samples.Rows() - 1));

		pos0[2] = -1;
		
		for (i = 0; i < samples.Rows() - 1; i++)
		{
			pos0[0] = -1;
			pos1 = pos0 + dz;
			
			for (j = 0; j < samples.Cols() - 1; j++)
			{
				pos3 = pos0 + dx;
				pos2 = pos1 + dx;
				pos0[1] = samples[i][j] + 0.01;
				pos1[1] = samples[i + 1][j] + 0.01;
				pos2[1] = samples[i + 1][j + 1] + 0.01;
				pos3[1] = samples[i][j + 1] + 0.01;
				
				normal = norm(cross(pos1 - pos0, pos3 - pos0)); 
				
				r.N(norm(normal));
				r.C(col).Begin(sRenderStyle)
					.P(pos0).P(pos1).P(pos2).P(pos3)
					.End();

				pos0 += dx;
				pos1 += dx;
			}
			
			pos0 += dz;
		}
	}
	else
	{
		dx = Vector(2.0 / samples.Cols(), 0, 0);
		dz = Vector(0, 0, 2.0 / samples.Rows());

		pos0[2] = -1;
		
		for (i = 0; i < samples.Rows(); i++)
		{
			pos0[0] = -1;
			pos1 = pos0 + dz;
			
			for (j = 0; j < samples.Cols(); j++)
			{
				pos3 = pos0 + dx;
				pos2 = pos1 + dx;
				pos0[1] = pos1[1] = pos2[1] = pos3[1] = samples[i][j];
			
				r.Begin(renPoly).P(pos0).P(pos1).P(pos2).P(pos3).End();
			
				pos0 += dx;
				pos1 += dx;
			}
			
			pos0 += dz;
		}
	}
}
  
