/*
	File:			SLLibrary.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1996, Andrew Willmott
*/


#include "gcl/SLLibrary.h"
#include "gcl/PolyLib.h"
#include "gcl/GCLConfig.h"
#include "gcl/MRModel.h"
#include <string.h>

/*! \class SLLibrary

	A library for scene objects: objects can be added and retrieved by
	name. Different libraries can be handed to scene construction /
	reading code in order to build scenes with an internal structure
	specific to a particular rendering / illumination method.

*/

// --- SLLibrary methods ------------------------------------------------------


SLLibrary::SLLibrary()
{
}

Void SLLibrary::Create()
{
	AddMember("group", new scGroup);
	AddMember("poly", new scPoly);
	AddMember("mr-model", new scMRModel);
	::AddPolyObjects();
	
	name = "simple";
}

Void SLLibrary::FreeData(Void *obj)
{
	scPrimitive *sp = (scPrimitive*) obj;

	delete sp;
}

Void SLLibrary::AddMember(scScenePtr objectPtr)
{
	SetVoidItem(objectPtr->Label(), (scPrimitive*) objectPtr);
}

Void SLLibrary::AddMember(StrConst name, scPrimitive *objectPtr)
{
	SetVoidItem(name, objectPtr);
}

scPrimitive *SLLibrary::Member(StrConst name)
{
	scPrimitive *sp = (scPrimitive*) GetVoidItem(name);

	if (!sp)
	{
		cerr << "Object not in library: " << name << endl;
		return(0);
	}
	else
		return(sp);
}

Bool SLLibrary::MemberExists(StrConst name)
{
	return(ItemExists(name));
}

Void SLLibrary::DeleteMember(StrConst name)
{
	DeleteItem(name);
}


// --- Output operators -------------------------------------------------------


class Printer : public HashIter
{
public: 
	Printer(ostream &s) : out(s) {};
	ostream &out;
	Void ProcessVoidItem(StrConst key, Void *data)
	{ out << key << endl; }
};

ostream &operator << (ostream &s, SLLibrary &library)
{
	Printer printer(s);

	library.Iterate(printer);

	return(s);
}
