/*
	File:			SLLibrary.h

	Function:		A library for scene objects: objects can be added and 
					retrieved by name. Different libraries can be handed to
					scene construction / reading code in order to build
					scenes with an internal structure specific to a particular
					rendering / illumination method.
					
	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 1995-1999, Andrew Willmott
 */


#ifndef __SLLibrary__
#define __SLLibrary__

#include "gcl/SceneObjects.h"
#include "cl/Hash.h"


class SLLibrary : public Hash
{
public:
	SLLibrary();
	
	virtual Void	Create();	
					///< Must be called before using the library

	Void			AddMember(scScenePtr objectPtr);
	Void			AddMember(StrConst name, scPrimitive *objectPtr);
	Bool			MemberExists(StrConst name);
	Void			DeleteMember(StrConst name);
	scPrimitive		*Member(StrConst name);
					/**< Return pointer to library member 'name'. You may want
						 to clone this before using it.*/
	
protected:
	Void			FreeData(Void *obj);
	String			name;
};

ostream &operator << (ostream &s, SLLibrary &library);

#endif
