/*
	File:		Texture.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/Texture.h"
#include "gcl/Geometry.h"

Texture::Texture(const Texture &tex)
{
	flags = tex.flags;
	textureFile = tex.textureFile;
	image = tex.image;
}

Int Texture::Load(StrConst str)
{
	Bool	bad = true;

	textureFile.SetPath(SubstituteEnvVars(str));
	image = new RGBAImage;
	
	if (image->Load(textureFile) != 0)
		_Warning(
			"Bad texture: not found");
	else if (image->Width() <= 0 || !IsPowerOfTwo(image->Width()))
		_Warning(
			"Bad texture: width is not a positive power of 2");
	else if (image->Height() <= 0 || !IsPowerOfTwo(image->Height()))
		_Warning(
			"Bad texture: height is not a positive power of 2");
	else
		bad = false;

	if (bad)
		// dummy up a texture
	{
		Int		i;
		image->SetSize(16, 16);
		image->Clear(cWhite);
		for (i = 0; i < 8; i++)
		{
			image->SetPixel(i, i, cRed);
			image->SetPixel(i, 15 - i, cRed);
			image->SetPixel(15 - i, i, cRed);
			image->SetPixel(15 - i, 15 - i, cRed);
		}
	}

	flags = 0;

	return(0);
}

#ifdef NEW
Colour Texture::Sample(Coord c)
{
	return(image->GetPixel(c[0] * (image->Width() - 1) + 0.5,
			c[1] * (image->Height() - 1) + 0.5));
}
#endif


Colour Texture::FilterBox(const Coord &bl, const Coord &ur)
{
	Int		x, y, lx, rx, uy, by;
	Colour	clrSum;
	
	lx = Int(image->Width() * bl[0] + 0.5);
	rx = Int(image->Width() * ur[0] + 0.5);
	by = Int(image->Height() * bl[1] + 0.5);
	uy = Int(image->Height() * ur[1] + 0.5);
	
	clrSum = vl_0;

	if (rx > lx && uy > by)
	{
		for (y = by; y < uy; y++)
			for (x = lx; x < rx; x++)
				clrSum += image->GetPixel(x, y);

		clrSum /= (uy - by) * (rx - lx);
	}

	return(clrSum);
}
