/*
	File:		TexTest.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include <iostream.h>
#include <GL/gl.h>
#include "XGraphicsSystem.h"

const Int checkImageWidth = 64;
const Int checkImageHeight = 64;
GLubyte checkImage[checkImageWidth][checkImageHeight][3];

void MakeCheckImage()
{
	int i, j, r, c;

	for (i = 0; i < checkImageWidth; i++)
		for (j = 0; j < checkImageHeight; j++)
		{
			c = ( ((i & 0x8) == 0) ^ ((j & 0x8) == 0) ) * 255;
			checkImage[i][j][0] = (GLubyte) c;
			checkImage[i][j][1] = (GLubyte) c;
			checkImage[i][j][2] = (GLubyte) c;
		}
}

void Init()
{
	MakeCheckImage();

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, checkImageWidth, checkImageHeight, 0,
				 GL_RGB, GL_UNSIGNED_BYTE, checkImage);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glShadeModel(GL_FLAT);
}

void TextureImage(Image &image)
{
	if (image.Tag() == imgRGBATag)
	{
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, image.Width(), image.Height(), 0,
					 GL_RGBA, GL_UNSIGNED_BYTE,
					 ((RGBAImage&) image).RGBAData());
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	}
	else
		_Error("non-rgba images unsupported");
}

void MyDisplay()
{
	glClearColor(0.0, 1.0, 0.0, 0.5);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glViewport(0, 0, 300, 300);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);

	glColor3f(1.0, 0.0, 1.0);
	glTexCoord2f(0.0, 0.0);
	glVertex3f(-0.8, -0.4, 0.0);

	glColor3f(1.0, 1.0, 0.0);
	glTexCoord2f(0.0, 1.0); 
	glVertex3f(0.8, -0.4, 0.0);

	glColor3f(0.0, 1.0, 1.0);
	glTexCoord2f(1.0, 1.0); 
	glVertex3f(0.8, 0.4, 0.0);

	glColor3f(0.0, 0.0, 1.0);
	glTexCoord2f(1.0, 0.0);
	glVertex3f(-0.8, 0.4, 0.0);
	glEnd();

	glDisable(GL_TEXTURE_2D);
	glColor3f(1.0, 1.0, 0.0);
	glBegin(GL_QUADS);
	glVertex3f(-0.8, -0.9, 0.0);
	glVertex3f(0.8, -0.9, 0.0);
	glVertex3f(0.8, -0.5, 0.0);
	glVertex3f(-0.8, -0.5, 0.0);
	glEnd();

	glFlush();
}

int main(int argc, char *argv[])
{
	XGraphicsSystem		xgs;
	GLXPane				pane;
//	GLXOffscreenPane	pane;
//	XBackedPane			display(&pane);
	RGBAImage			tex;

	if (argc > 1)
		tex.Load(FileName().SetPath(argv[1]));
	else
		tex.Load(FileName().SetPath(
			"$HOME/images/grey.tif"
			));
	xgs.CreateWindow(&pane, "texture test", 300, 300);
	pane.SetBgColour(cRed);
	pane.Clear().Show();
	pane.MakeCurrent();
	TextureImage(tex);
	MyDisplay();
	xgs.Run();

	return(0);
}
